@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: prefs.h,v 1.11 2007/04/30 15:50:39 leonb Exp $
-// $Name: release_3_5_20_2 $
 
 #ifndef HDR_PREFS
 #define HDR_PREFS
@@ -88,8 +85,8 @@ public:
    static const int legal_mag_scale[];
    static const int legal_mag_size_num;
    static const int legal_mag_scale_num;
-   static char	* hlb_names[];
-   static char	* mag_names[];
+   static const char *hlb_names[];
+   static const char *mag_names[];
 
    char		bBeginner;
 
