@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: GSmartPointer.h,v 1.15 2007/12/02 15:13:17 leonb Exp $
-// $Name: release_3_5_20_2 $
 
 #ifndef _GSMARTPOINTER_H_
 #define _GSMARTPOINTER_H_
@@ -87,8 +84,6 @@
 // Our original implementation consisted of multiple classes.
 // <http://prdownloads.sourceforge.net/djvu/DjVu2_2b-src.tgz>.
 
-    @version 
-    #$Id: GSmartPointer.h,v 1.15 2007/12/02 15:13:17 leonb Exp $# 
     @args
 */
 //@{
@@ -100,6 +95,7 @@
 #endif
 
 #include "DjVuGlobal.h"
+#include "atomic.h"
 
 #ifdef HAVE_NAMESPACES
 namespace DJVU {
@@ -117,6 +113,10 @@ namespace DJVU {
  */
 class DJVUAPI GPEnabled
 {
+  friend class GPBase;
+  void destroy();
+  void unref();
+  void ref();
 public:
   /// Null constructor.
   GPEnabled();
@@ -132,11 +132,6 @@ public:
 protected:
   /// The reference counter
   volatile int count;
-private:
-  friend class GPBase;
-  void unref();
-  void ref();
-  void destroy();
 };
 
 
@@ -205,8 +200,10 @@ protected:
     The first time you use a smart-pointer to access #GPEnabled# object, the
     reference counter is incremented to one. Object destruction will then
     happen automatically when the reference counter is decremented back to
-    zero (i.e. when the last smart-pointer referencing this object stops doing so).
-    This will happen regardless of how many regular pointers reference this object.
+    zero (i.e. when the last smart-pointer referencing 
+    this object stops doing so).
+    This will happen regardless of how many regular pointers 
+    reference this object.
     In other words, if you start using smart-pointers with a #GPEnabled#
     object, you engage automatic mode for this object.  You should only do
     this with objects dynamically allocated with operator #new#.  You should
@@ -332,6 +329,25 @@ GPEnabled::operator=(const GPEnabled & obj)
   return *this; 
 }
 
+inline void 
+GPEnabled::ref()
+{
+#if PARANOID_DEBUG
+  assert (count >= 0);
+#endif
+  atomicIncrement(&count);
+}
+
+inline void 
+GPEnabled::unref()
+{
+#if PARANOID_DEBUG
+  assert (count > 0);
+#endif
+  if (! atomicDecrement(&count))
+    destroy();
+}
+
 // INLINE FOR GPBASE
 
 inline
