@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: MMX.h,v 1.11 2007/05/18 19:50:41 leonb Exp $
-// $Name: release_3_5_23 $
 
 #ifndef _MMX_H_
 #define _MMX_H_
@@ -106,8 +103,6 @@ namespace DJVU {
 
     @memo
     Essential support for MMX.
-    @version 
-    #$Id: MMX.h,v 1.11 2007/05/18 19:50:41 leonb Exp $#
     @author: 
     L\'eon Bottou <leonb@research.att.com> -- initial implementation */
 //@{
