@@ -15,7 +15,6 @@
 // GNU General Public License for more details.
 // -------------------------------------------------------------------
 */
-/* $Id: miniexp.h,v 1.18 2007/12/02 03:58:29 leonb Exp $ */
 
 #ifndef MINIEXP_H
 #define MINIEXP_H
@@ -294,7 +293,10 @@ MINILISPAPI miniexp_t miniexp_concat(miniexp_t l);
    can reference an arbitrary lisp expression.  Garbage
    collection preserves all lisp expressions referenced by a
    minivar, as well as all lisp expressions that can be
-   accessed from these.
+   accessed from these. When called automatically, 
+   garbage collection also preserves the sixteen most recently 
+   created miniexps in order to make sure that temporaries do 
+   not vanish in the middle of complicated C expressions.
      
    The minivar class is designed such that C++ program can
    directly use instances of <minivar_t> as normal
@@ -418,7 +420,7 @@ MINILISPAPI void minilisp_debug(int debugflag);
 /* minilisp_finish --
    Deallocates everything.  This is only useful when using
    development tools designed to check for memory leaks.  
-   No miniexp function can be used after calliang this. */
+   No miniexp function can be used after calling this. */
 
 MINILISPAPI void minilisp_finish(void);
 
@@ -427,7 +429,7 @@ MINILISPAPI void minilisp_finish(void);
 /* INPUT/OUTPUT                                       */
 /* -------------------------------------------------- */
 
-/* Notes about the textual represenation of miniexps.
+/* Notes about the textual representation of miniexps.
 
    - Special characters are:
      * the parenthesis <(> and <)>,
@@ -440,7 +442,7 @@ MINILISPAPI void minilisp_finish(void);
      Vertical bars <|> can be used to delimit names that
      contain blanks, special characters, non printable
      characters, non ascii characters, or 
-     can be confused as a number.
+     can be confused for a number.
      
    - Numbers follow the syntax specified by the C
      function strtol() with base=0.
@@ -593,13 +595,16 @@ miniobj_t {
   virtual bool isa(miniexp_t classname) const;
 
   /* --- optional stuff --- */
-  /* mark: iterates over miniexps contained by this object
-     for garbage collecting purposes. */
-  virtual void mark(minilisp_mark_t*);
-
   /* pname: returns a printable name for this object.
      The caller must deallocate the result with delete[]. */
   virtual char *pname() const;
+  /* mark: iterates over miniexps contained by this object
+     for garbage collecting purposes. */
+  virtual void mark(minilisp_mark_t*);
+  /* destroy: called by the garbage collector to
+     deallocate the object. Defaults to 'delete this'. */
+  virtual void destroy();
+     
 };
 
 /* MINIOBJ_DECLARE --
